"""Qlabel module."""

# -*- coding: utf-8 -*-

from PyQt6 import QtWidgets, QtGui  # type: ignore[import]
from typing import Any, Union, Optional

from pineboolib.core import decorators


class QLabel(QtWidgets.QLabel):
    """QLabel class."""

    def __init__(
        self,
        text_or_parent: Union[str, QtWidgets.QWidget],
        parent: Optional[QtWidgets.QWidget] = None,
        name: Optional[str] = None,
    ):
        """Inititalize."""

        super().__init__(text_or_parent)

        if parent is not None:
            self.setParent(parent)

        if name is not None:
            self.setObjectName(name)

    def get_text(self) -> str:
        """Return text label."""

        return super().text()

    def setText(self, text: Union[str, int]) -> None:  # type: ignore [override]
        """Set text label."""

        if not isinstance(text, str):
            text = str(text)
        super().setText(text)

    def setPixmap(self, pix: Union[QtGui.QIcon, QtGui.QPixmap]) -> None:
        """Set pixmap."""

        if isinstance(pix, QtGui.QIcon):
            pix = pix.pixmap(32, 32)
        super(QLabel, self).setPixmap(pix)

    @decorators.pyqt_slot(bool)
    def setShown(self, visible: bool):
        """Set visible."""

        self.setVisible(visible)

    def getAlign(self) -> Any:
        """Return Alignment."""

        return super().alignment()

    def setAlign(self, alignment_: Any) -> None:
        """Set alignment."""

        self.setAlignment(alignment_)

    def get_palette_fore_ground(self) -> QtGui.QColor:
        """Return palette foreground color."""

        return self.palette().text().color()

    def set_palette_fore_ground(self, color: QtGui.QColor) -> None:
        """Set palette foreground color."""
        pal = self.palette()
        pal.setColor(pal.ColorRole.WindowText, color)
        self.setPalette(pal)

    alignment = property(getAlign, setAlign)  # type: ignore
    text = property(get_text, setText)  # type: ignore
    paletteForegroundColor = property(get_palette_fore_ground, set_palette_fore_ground)



"""
Utilities for loading pineboo.
"""
import traceback
from pineboolib import logging
from typing import Dict, Any, Callable

LOGGER = logging.get_logger(__name__)


def monkey_patch_connect() -> None:
    """
    Patch Qt6 signal/event functions for tracing them.

    This is not stable and should be used with care
    """
    from PyQt6 import QtCore  # type: ignore

    LOGGER.warning(
        "--trace-signals es experimental. Tiene problemas de memoria y falla en llamadas con un argumento (False)"
    )
    LOGGER.warning(
        "... se desaconseja su uso excepto para depurar. Puede cambiar el comportamiento del programa."
    )

    class BoundSignal:
        """BoundSignal class."""

        _CONNECT = QtCore.pyqtBoundSignal.connect  # type: ignore
        _EMIT = QtCore.pyqtBoundSignal.emit
        _LAST_EMITTED_SIGNAL: Dict[str, Any] = {}

        def slot_decorator(self: Any, slot: Callable, connect_stack: Any) -> Callable:
            """Create Slot decorator."""

            selfid = repr(self)

            def decorated_slot(*args: Any) -> Any:
                """Return decorated slot."""

                ret = None
                if len(args) == 1 and args[0] is False:
                    args = tuple()
                try:
                    # print("Calling slot: %r %r" % (slot, args))
                    ret = slot(*args)
                except Exception:
                    LOGGER.error("Unhandled exception in slot %r (%r): %r" % (slot, self, args))
                    LOGGER.error("-- Connection --")
                    LOGGER.error(traceback.format_list(connect_stack)[-2].rstrip())
                    last_emit_stack = BoundSignal._LAST_EMITTED_SIGNAL.get(selfid, None)
                    if last_emit_stack:
                        LOGGER.error("-- Last signal emmitted --")
                        LOGGER.error(traceback.format_list(last_emit_stack)[-2].rstrip())
                    LOGGER.error("-- Slot traceback --")
                    LOGGER.error(traceback.format_exc())
                return ret

            return decorated_slot

        def connect(self: Any, slot: Any, type_: int = 0, no_receiver_check: bool = False) -> Any:
            """
            Proxy a connection to the original connect in the Qt library.

            This function wraps on top of the original Qt.connect so everything
            is logged.

            slot is either a Python callable or another signal.
            type is a Qt.ConnectionType. (default Qt.AutoConnection = 0)
            no_receiver_check is True to disable the check that the receiver's C++
            instance still exists when the signal is emitted.
            """
            clname = getattr(getattr(slot, "__class__", {}), "__name__", "not a class")
            # print("Connect: %s -> %s" % (type(self), slot))
            if clname == "method":
                stack = traceback.extract_stack()
                newslot = BoundSignal.slot_decorator(self, slot, stack)
            else:
                newslot = slot
            return BoundSignal._CONNECT(self, newslot)  # type: ignore [call-arg] # noqa: F821

        def emit(self: Any, *args: Any) -> Any:
            """Proxy original Qt Emit function for tracing signal emits."""
            # print("Emit: %s :: %r" % (self, args))
            stack = traceback.extract_stack()
            # print(traceback.format_list(stack)[-2].rstrip())
            BoundSignal._LAST_EMITTED_SIGNAL[repr(self)] = stack
            return BoundSignal._EMIT(self, *args)

    QtCore.pyqtBoundSignal.connect = BoundSignal.connect  # type: ignore
    QtCore.pyqtBoundSignal.emit = BoundSignal.emit  # type: ignore



"""Flmanagermodules module."""

# -*- coding: utf-8 -*-
from PyQt6 import QtWidgets, QtGui  # type: ignore[import]

from pineboolib.core import decorators
from pineboolib.core.utils import utils_base
from pineboolib.core import settings

from pineboolib.application.metadata import pnaction
from pineboolib.application.staticloader import pnmodulesstaticloader


from pineboolib.application.utils import path, xpm, convert_flaction


from pineboolib import application
from pineboolib.application.utils.path import _path


from pineboolib.fllegacy import flutil
from pineboolib.fllegacy import flformdb
from pineboolib.fllegacy import flformrecorddb

from pineboolib import logging

from typing import Union, List, Dict, Optional, cast, Any, TYPE_CHECKING
from watchdog import observers, events  # type: ignore [import] # noqa: F821
import os
import codecs

from xml.etree import ElementTree as ET

if TYPE_CHECKING:
    from pineboolib.application import xmlaction  # noqa: F401 # pragma: no cover
    from pineboolib.interfaces import iconnection, isqlcursor  # noqa : F401 # pragma: no cover


"""
Gestor de módulos.

Esta clase permite realizar las funciones básicas de manejo de ficheros
de texto que forman parte de los módulos de aplicación, utilizando como
soporte de almacenamiento la base de datos y el sistema de cachés de texto
para optimizar las lecturas.
Gestiona la carga y descarga de módulos. Mantiene cual es el módulo activo.
El módulo activo se puede establecer en cualquier momento con
FLManagerModules::setActiveIdModule().

Los módulos se engloban en áreas (FACTURACION, FINANCIERA, PRODUCCION, etc..) y
cada módulo tiene varios ficheros de texto XML y scripts. Toda la estructura de
módulos se almacena en las tablas flareas, flmodulos, flserial y flfiles, sirviendo esta
clase como interfaz para el manejo de dicha estructura en el entorno de trabajo
de AbanQ.

@author InfoSiAL S.L.
"""

LOGGER = logging.get_logger(__name__)


class FLManagerModules(object):
    """FLManagerModules class."""

    """
    Mantiene el identificador del módulo activo.
    """
    active_id_module_: str

    """
    Mantiene la clave sha correspondiente a la version de los módulos cargados localmente
    """
    sha_local_: Optional[str]

    """
    Diccionario de claves de ficheros, para optimizar lecturas
    """
    dict_key_files_: Dict[str, str]

    """
    Diccionario de identificadores de modulo de ficheros, para optimizar lecturas
    """
    dict_module_files_: Dict[str, str]

    """
    Uso interno.
    Informacion para la carga estatica desde el disco local
    """
    static_db_info_: "pnmodulesstaticloader.AQStaticBdInfo"
    _file_watcher: "observers.Observer"  # type: ignore [valid-type]
    root_dir_: str
    scripts_dir_: str
    tables_dir_: str
    forms_dir_: str
    reports_dir_: str
    queries_dir_: str
    trans_dir_: str
    _files_cached: Dict[str, str]

    def __init__(self, db_: "iconnection.IConnection") -> None:
        """Inicialize."""

        if db_ is None:
            raise ValueError("Database is required")
        self.conn_ = db_

        self.commonInit()
        self.active_id_module_ = ""
        self.active_id_area_ = ""
        self.sha_local_ = ""
        self._files_cached = {}
        self.dict_key_files_ = {}

        self.dict_module_files_ = {}

    def commonInit(self) -> None:
        """Run common init."""

        self.static_db_info_ = pnmodulesstaticloader.AQStaticBdInfo(self.conn_)
        self._file_watcher = observers.Observer()
        flfiles_folder = application.PROJECT.USE_FLFILES_FOLDER

        if self.static_db_info_.enabled_ or flfiles_folder:
            num_folders = 0
            LOGGER.warning("STATIC LOAD IS ENABLED!")
            event_handler = events.FileSystemEventHandler()

            if flfiles_folder and application.USE_FLFILES_FOLDER_AS_STATIC_LOAD:
                LOGGER.warning("USING FLFILES_FOLDER AS STATIC LOAD FOLDER!")
                num_folders += 1 if self.addFolder(flfiles_folder, event_handler) else 0

            self.static_db_info_.readSettings()
            for dir_path in self.static_db_info_.dirs_:
                LOGGER.warning("STATIC LOAD: %s IS %s", dir_path.path_, dir_path.active_)
                if dir_path.active_:
                    num_folders += 1 if self.addFolder(dir_path.path_, event_handler) else 0

            if num_folders > 0:
                event_handler.on_any_event = self.static_db_info_.msg_static_changed  # type: ignore [assignment]

                self._file_watcher.start()
                LOGGER.warning("STATIC LOAD IS WORKING")

    def addFolder(self, folder: str, event_handler) -> bool:
        """Add folder."""

        if os.path.exists(folder):
            self._file_watcher.schedule(event_handler, folder, recursive=True)  # type: ignore [attr-defined]
        else:
            LOGGER.warning("STATIC LOAD: %s FOLDER DOESN'T EXISTS !" % folder)
            return False

        return True

    def reloadStaticLoader(self) -> None:
        """Reload static loader."""
        pnmodulesstaticloader.SHOW_REINIT_MESSAGE = True
        if hasattr(self, "_file_watcher"):
            self._file_watcher.stop()  # type: ignore [attr-defined]
            del self._file_watcher
        del self.static_db_info_

        self.commonInit()

    def finish(self) -> None:
        """Run tasks when closing the module."""

        del self.dict_module_files_
        self.dict_module_files_ = {}

        del self.static_db_info_
        self.static_db_info_ = pnmodulesstaticloader.AQStaticBdInfo(self.conn_)

        self.writeState()
        del self.dict_key_files_
        self.dict_key_files_ = {}

        del self._files_cached
        self._files_cached = {}

    def content(self, file_name: str) -> str:
        """
        Get the contents of a file stored in the database.

        This method looks for the content of the requested file in the
        database, exactly in the flfiles table, if you can't find it
        Try to get it from the file system.

        @param file_name File name.
        @return QString with the contents of the file or empty in case of error.
        """

        result_conn = (
            self.conn_.connManager()
            .dbAux()
            .execute_query(
                "SELECT contenido FROM flfiles WHERE nombre='%s' AND NOT sha = ''" % file_name
            )
        )
        if result_conn is not None:
            ret = result_conn.first()
            if ret is not None:
                return ret[0]

        return ""

    @decorators.not_implemented_warn
    def byteCodeToStr(self, file_name: str) -> str:
        """
        Get the contents of a script file.

        Get the contents of a script file, processing it to change the connections it contains,
        so that at the end of the execution of the connected function the test script resumes.
        It also performs code formatting processes to optimize it.

        @param file_name File name.
        @return QString with the contents of the file or empty in case of error.
        """
        return ""

    @decorators.not_implemented_warn
    def contentCode(self, file_name: str) -> str:
        """
        Return the contents of a script file.

        Return the contents of a script file processing it to change the connections it contains,
        so that at the end of the execution of the connected function the test script resumes.
        It also performs code formatting processes to optimize it.

        @param file_name File name.
        @return QString with the contents of the file or empty in case of error.
        """
        return ""

    def contentFS(self, path_name: str, utf8: bool = False) -> str:
        """
        Return the contents of a file stored in the file system.

        @param path_name Path and file name in the file system
        @return QString with the contents of the file or empty in case of error.
        """
        encode_ = "UTF-8" if utf8 else "ISO-8859-15"

        try:
            return str(open(path_name, "rb").read(), encode_)
        except Exception:
            LOGGER.warning("Error trying to read %r", path_name, exc_info=True)
            return ""

    def contentCached(self, file_name: str, sha_key=None) -> Optional[str]:
        """
        Get the contents of a file, using the memory and disk cache.

        This method first looks for the content of the requested file in the
        Internal cache, if not, you get it with the FLManagerModules :: content () method.

        @param file_name File name.
        @return QString with the contents of the file or None in case of error.
        """

        sys_table: bool = (
            self.conn_.connManager().manager().isSystemTable(file_name)
            if file_name.endswith(".mtd")
            else False
        )

        data = ""

        if not sys_table and self.static_db_info_ and self.static_db_info_.enabled_:
            data = self.contentStatic(file_name)

        if not data:
            if file_name in self._files_cached.keys():
                data = self._files_cached[file_name]
            else:
                path_file = _path(file_name, False)
                if path_file is not None and os.path.exists(path_file):
                    file_encode = "UTF8" if file_name.endswith((".ts", ".py")) else "ISO-8859-15"

                    file_ = codecs.open(path_file, "r", file_encode)
                    data = file_.read()
                    file_.close()

                elif not application.PROJECT.USE_FLFILES_FOLDER:  # load from database
                    data = self.content(file_name)

        if data:
            self._files_cached[file_name] = data

        return data

    def setContent(self, file_name: str, id_module: str, content: str) -> None:
        """
        Store the contents of a file in a given module.

        @param file_name File name.
        @param id_module Identifier of the module to which the file will be associated
        @param content File content.
        """

        from pineboolib.application.database import pnsqlcursor

        format_val = (
            self.conn_.connManager()
            .manager()
            .formatAssignValue("nombre", "string", file_name, True)
        )
        format_val2 = (
            self.conn_.connManager()
            .manager()
            .formatAssignValue("idmodulo", "string", id_module, True)
        )

        cursor = pnsqlcursor.PNSqlCursor("flfiles", True, "dbAux")
        cursor.setActivatedCheckIntegrity(False)
        cursor.select("%s AND %s" % (format_val, format_val2))

        if cursor.first():
            cursor.setModeAccess(cursor.Edit)
            cursor.refreshBuffer()
        else:
            cursor.setModeAccess(cursor.Insert)
            cursor.refreshBuffer()
            cursor.setValueBuffer("nombre", file_name)
            cursor.setValueBuffer("idmodulo", id_module)

        cursor.setValueBuffer("contenido", content)
        cursor.setValueBuffer("sha", flutil.FLUtil().sha1(content))
        cursor.commitBuffer()

    @staticmethod
    def createUI(
        file_name: str,
        connection: Optional["iconnection.IConnection"] = None,
        parent: Optional["QtWidgets.QWidget"] = None,
    ) -> Optional["QtWidgets.QWidget"]:
        """
        Create a form from its description file.

        Use the FLManagerModules :: contentCached () method to get the XML text it describes the formula.

        @param file_name Name of the file that contains the description of the form.
        @param parent. Parent widget
        @return QWidget corresponding to the built form.
        """

        if ".ui" not in file_name:
            file_name += ".ui"

        form_path = file_name if os.path.exists(file_name) else path._path(file_name, False)
        conn_manager = application.PROJECT.conn_manager

        if "main_conn" in conn_manager.connections_dict.keys():
            mng_modules = conn_manager.managerModules()
            if mng_modules.static_db_info_ and mng_modules.static_db_info_.enabled_:
                static_path = mng_modules.contentStatic(file_name, True)
                if static_path:
                    form_path = static_path

        if not form_path:
            # raise AttributeError("File %r not found in project" % n)
            LOGGER.warning("createUI: No se encuentra el fichero %s", file_name)

            return None

        tree = utils_base.load2xml(form_path)

        if not tree:
            return parent or QtWidgets.QWidget()

        root_ = tree.getroot()

        ui_version = root_.get("version") or "1.0"
        wid = root_.find("widget")
        geometry = []

        if wid is not None:
            for prop in wid.findall("property"):
                if prop.get("name") == "geometry":
                    geometry.append(
                        prop.find("rect")  # type: ignore [union-attr] # noqa: F821
                        .find("width")  # type: ignore [union-attr] # noqa: F821
                        .text
                    )
                    geometry.append(
                        prop.find("rect")  # type: ignore [union-attr] # noqa: F821
                        .find("height")  # type: ignore [union-attr] # noqa: F821
                        .text
                    )
                    break

        if parent is None:
            if wid is None:
                raise Exception("No parent provided and also no <widget> found")
            xclass = wid.get("class")

            if xclass is None:
                raise Exception("class was expected")

            if ui_version < "4.0":
                if xclass == "QMainWindow":
                    from pineboolib.q3widgets import qmainwindow

                    parent = qmainwindow.QMainWindow()
                elif xclass in ["QDialog", "QWidget"]:
                    from pineboolib.q3widgets import qdialog

                    parent = qdialog.QDialog()
            else:
                if xclass == "QMainWindow":
                    parent = QtWidgets.QMainWindow()
                elif xclass in ["QDialog", "QWidget"]:
                    parent = QtWidgets.QDialog()

            if parent is None:
                raise Exception("xclass not found %s" % xclass)

        LOGGER.info("Procesando %s (v%s)", file_name, ui_version)
        if ui_version < "4.0":
            from pineboolib.application.parsers.parser_ui import qt3ui

            qt3ui.load_ui(form_path, parent)
        else:
            from PyQt6 import uic  # type: ignore

            qt_widgets_path = utils_base.filedir("plugins/custom_widgets")
            if qt_widgets_path not in uic.widgetPluginPath:
                LOGGER.info("Añadiendo path %s a uic.widgetPluginPath", qt_widgets_path)
                uic.widgetPluginPath.append(qt_widgets_path)

            uic.loadUi(form_path, parent)

        if geometry[0]:
            form_parent: Any = parent
            if parent.parent() is not None:
                form_parent = parent.parent()
            form_parent.resize(
                int(geometry[0]), int(geometry[1])  # type: ignore [arg-type] # noqa: F821
            )
        return parent

    def createForm(
        self,
        action: Union["pnaction.PNAction", "xmlaction.XMLAction"],
        connector: Optional["iconnection.IConnection"] = None,
        parent: Optional["QtWidgets.QWidget"] = None,
        name: Optional[str] = None,
    ) -> "flformdb.FLFormDB":
        """
        Create the master form of an action from its description file.

        Use the FLManagerModules :: createUI () method to get the built form.

        @param to FLAction Object.
        @return QWidget corresponding to the built form.
        """

        if not isinstance(action, pnaction.PNAction):
            action = convert_flaction.convert_to_flaction(action)

        if action is None:
            raise Exception("action is empty!.")

        # if parent is None:
        #    from pineboolib.fllegacy import flapplication

        #    parent = flapplication.aqApp.mainWidget()

        return flformdb.FLFormDB(action, parent, load=True)

    def createFormRecord(
        self,
        action: Union["pnaction.PNAction", "xmlaction.XMLAction"],
        connector: Optional["iconnection.IConnection"] = None,
        parent_or_cursor: Optional[Union["isqlcursor.ISqlCursor", "QtWidgets.QWidget"]] = None,
        name: Optional[str] = None,
    ) -> "flformrecorddb.FLFormRecordDB":
        """
        Create the record editing form of an action from its description file.

        @param action. Action
        @param connector. Connector used
        @param parent_or_cursor. Cursor or parent of the form
        @param name. FormRecord name
        """

        LOGGER.trace("createFormRecord: init")

        # Falta implementar conector y name
        if not isinstance(action, pnaction.PNAction):
            LOGGER.trace("createFormRecord: convert2FLAction")

            action = convert_flaction.convert_to_flaction(action)

        if action is None:
            raise Exception("action is empty!")

        LOGGER.trace("createFormRecord: load FormRecordDB")
        return flformrecorddb.FLFormRecordDB(action, parent_or_cursor, load=False)

    def setActiveIdModule(self, id_module: str = "") -> None:
        """
        Set the active module.

        It also automatically establishes the area corresponding to the module,
        since a module can only belong to a single area.

        @param id_module Module identifier
        """

        self.active_id_module_ = (
            id_module if id_module in application.PROJECT.modules.keys() else ""
        )

    def activeIdArea(self) -> str:
        """
        Return the area of the active module.

        @return Area identifier
        """

        return (
            application.PROJECT.modules[self.active_id_module_].areaid
            if self.active_id_module_ in application.PROJECT.modules
            else ""
        )

    def activeIdModule(self) -> str:
        """
        Return the active module.

        @return Module identifier
        """

        return self.active_id_module_

    def listIdAreas(self) -> List[str]:
        """
        Return the list of area identifiers loaded in the system.

        @return List of area identifiers
        """

        return list(application.PROJECT.areas.keys())

    def listIdModules(self, id_area: str) -> List[str]:
        """
        Return the list of module identifiers loaded into the system of a given area.

        @param id_area Identifier of the area from which you want to get the modules list
        @return List of module identifiers
        """

        return [
            key
            for key in application.PROJECT.modules.keys()
            if application.PROJECT.modules[key].areaid == id_area
        ]

    def listAllIdModules(self) -> List[str]:
        """
        Return the list of identifiers of all modules loaded in the system.

        @return List of module identifiers
        """

        return [key for key in application.PROJECT.modules.keys()]

    def idAreaToDescription(self, id_area: str = "") -> str:
        """
        Return the description of an area from its identifier.

        @param id_area Area identifier.
        @return Area description text, if found or idA if not found.
        """

        return (
            application.PROJECT.areas[id_area].descripcion
            if id_area in application.PROJECT.areas.keys()
            else ""
        )

    def idModuleToDescription(self, id_module: str = "") -> str:
        """
        Return the description of a module from its identifier.

        @param id_module Module identifier.
        @return Module description text, if found or idM if not found.
        """

        return (
            application.PROJECT.modules[id_module].description
            if id_module in application.PROJECT.modules.keys()
            else ""
        )

    def iconModule(self, id_module: str) -> "QtGui.QPixmap":
        """
        To obtain the icon associated with a module.

        @param id_moule Identifier of the module from which to obtain the icon
        @return QPixmap with the icon
        """

        return (
            QtGui.QPixmap(xpm.cache_xpm(application.PROJECT.modules[id_module].icon))
            if id_module in application.PROJECT.modules.keys()
            else QtGui.QPixmap()
        )

    def versionModule(self, id_module: str) -> str:
        """
        Return the version of a module.

        @param id_module Identifier of the module whose version you want to know
        @return Chain with version
        """

        return (
            application.PROJECT.modules[id_module].version
            if id_module in application.PROJECT.modules.keys()
            else id_module
        )

    def shaLocal(self) -> Optional[str]:
        """
        To obtain the local sha key.

        @return Sha key of the locally loaded modules version
        """

        return self.sha_local_

    def shaGlobal(self) -> str:
        """
        To get the global sha key.

        @return Sha key of the globally loaded modules version
        """

        if not self.conn_.connManager().dbAux():
            return ""

        from pineboolib.application.database import pnsqlquery

        qry = pnsqlquery.PNSqlQuery(None, "dbAux")
        qry.setForwardOnly(True)
        qry.exec_("SELECT sha FROM flserial")
        if qry.lastError is None:
            return "error"

        return str(qry.value(0)) if qry.next() else ""

    def setShaLocalFromGlobal(self) -> None:
        """
        Set the value of the local sha key with that of the global one.
        """

        self.sha_local_ = self.shaGlobal()

    def shaOfFile(self, file_name: str) -> str:
        """
        Get the sha key associated with a stored file.

        @param file_name File name
        @return Key sh associated with the files
        """
        ret_ = ""
        if not self.conn_.connManager().manager().isSystemTable(file_name):
            if file_name in application.PROJECT.files.keys():
                ret_ = application.PROJECT.files[file_name].sha or ""

        return ret_

    def loadKeyFiles(self) -> None:
        """
        Load the sha1 keys of the files into the key dictionary.
        """

        self.dict_key_files_ = {}
        self.dict_module_files_ = {}

        from pineboolib.application.database import pnsqlquery

        qry = pnsqlquery.PNSqlQuery(None, "dbAux")
        # qry.setForwardOnly(True)
        qry.exec_("SELECT nombre, sha, idmodulo FROM flfiles")

        while qry.next():
            name = str(qry.value(0))
            self.dict_key_files_[name] = str(qry.value(1))
            self.dict_module_files_[name.upper()] = str(qry.value(2))

    @decorators.deprecated
    def loadAllIdModules(self) -> None:
        """
        Load the list of all module identifiers.
        """

        # =======================================================================
        # self.dict_info_mods_ = {}
        # for id_module in application.PROJECT.modules.keys():
        #     info_module_ = pninfomod.PNInfoMod()
        #     info_module_.id_modulo = id_module
        #     info_module_.id_area = application.PROJECT.modules[id_module].areaid
        #     info_module_.descripcion = application.PROJECT.modules[id_module].description
        #     info_module_.version = ""
        #     info_module_.icono = application.PROJECT.modules[id_module].icon
        #     info_module_.area_descripcion = application.PROJECT.areas[
        #         application.PROJECT.modules[id_module].areaid
        #     ].descripcion
        #     self.dict_info_mods_[info_module_.id_modulo.upper()] = info_module_
        # =======================================================================

    def listAllIdModules(self) -> List[str]:
        """
        Return the list of identifiers of all modules loaded in the system.

        @return List of module identifiers
        """

        return [key for key in application.PROJECT.modules.keys()]

    @decorators.deprecated
    def loadIdAreas(self) -> None:
        """
        Load the list of all area identifiers.
        """

        pass
        # for key in application.PROJECT.areas.keys():
        #    self.dict_areas[key] = application.PROJECT.areas[key].descripcion

    @decorators.not_implemented_warn
    def checkSignatures(self):
        """
        Check the signatures for a given module.
        """

        pass

    def idModuleOfFile(self, name: str = "") -> str:
        """
        Return the identifier of the module to which a given file belongs.

        @param n File name including extension
        @return Identifier of the module to which the file belongs
        """
        if name.endswith(".mtd"):
            if application.PROJECT.conn_manager.manager().isSystemTable(name):
                return "sys"
            elif "%s_model.py" % name[:-4] in application.PROJECT.files.keys():
                return application.PROJECT.files["%s_model.py" % name[:-4]].module

        elif name in application.PROJECT.files.keys():
            return application.PROJECT.files[name].module
        else:
            LOGGER.warning(
                "Can't found %s ** %s", name, application.PROJECT.files.keys(), stack_info=True
            )

        return ""

    def writeState(self) -> None:
        """
        Save the status of the module system.
        """

        id_db = "noDB"
        db_aux = self.conn_.connManager().dbAux()
        if db_aux:
            id_db = "%s%s%s%s%s" % (
                db_aux.database(),
                db_aux.host(),
                db_aux.user(),
                db_aux.driverName(),
                db_aux.port(),
            )

        if self.active_id_area_ is None:
            self.active_id_area_ = ""

        if self.active_id_module_ is None:
            self.active_id_module_ = ""

        if self.sha_local_ is None:
            raise ValueError("sha_local_ is empty!")

        settings.SETTINGS.setValue("Modules/activeIdModule/%s" % id_db, self.active_id_module_)  # type: ignore [has-type]
        settings.SETTINGS.setValue("Modules/activeIdArea/%s" % id_db, self.active_id_area_)  # type: ignore [has-type]
        settings.SETTINGS.setValue("Modules/shaLocal/%s" % id_db, self.sha_local_)  # type: ignore [has-type]

    def readState(self) -> None:
        """
        Read the module system status.
        """
        db_aux = self.conn_.connManager().dbAux()

        if db_aux:
            id_db = "%s%s%s%s%s" % (
                db_aux.database(),
                db_aux.host(),
                db_aux.user(),
                db_aux.driverName(),
                db_aux.port(),
            )

            self.active_id_module_ = settings.SETTINGS.value(
                "Modules/activeIdModule/%s" % id_db, None
            )
            self.active_id_area_ = settings.SETTINGS.value("Modules/activeIdArea/%s" % id_db, None)
            self.sha_local_ = settings.SETTINGS.value("Modules/shaLocal/%s" % id_db, None)

            if (
                self.active_id_module_ is None
                or self.active_id_module_ not in self.listAllIdModules()
            ):
                self.active_id_module_ = ""

    def contentStatic(self, file_name: str, only_path: bool = False) -> str:
        """
        Return the contents of a file by static loading from the local disk.

        @param file_name File name.
        @return String with the contents of the file or None in case of error.
        """

        str_ret = pnmodulesstaticloader.PNStaticLoader.content(
            file_name, self.static_db_info_, only_path
        )
        if str_ret:
            mng = application.PROJECT.conn_manager.manager()
            text_ = ""
            util = flutil.FLUtil()
            sha = util.sha1(str_ret)
            if file_name in self.dict_key_files_.keys():
                text_ = self.dict_key_files_[file_name]

            if text_ == sha:
                return ""

            elif self.dict_key_files_ and file_name.find(".qs") > -1:
                self.dict_key_files_[file_name] = sha

            if file_name.endswith(".mtd"):
                mtd = mng.metadata(ET.fromstring(str_ret), True)

                if mtd is not None and not mtd.isQuery():
                    conn_ = application.PROJECT.conn_manager.useConn("dbaux")
                    if not conn_.existsTable(mtd.name()):
                        conn_.createTable(mtd)
                    elif conn_.canRegenTables():
                        conn_.regenTable(mtd.name(), mtd)

        return str_ret

    def staticLoaderSetup(self) -> None:
        """
        Display dialog box to configure static load from local disk.
        """
        ui_ = cast(
            QtWidgets.QDialog,
            self.createUI(utils_base.filedir("./application/staticloader/ui/static_loader.ui")),
        )
        pnmodulesstaticloader.PNStaticLoader.setup(self.static_db_info_, ui_)



"""Qvboxlayout module."""
# -*- coding: utf-8 -*-
from PyQt6 import QtWidgets  # type: ignore[import]
from typing import Optional


class QVBoxLayout(QtWidgets.QVBoxLayout):
    """QVBoxLayout class."""

    def __init__(self, parent: Optional[QtWidgets.QWidget] = None) -> None:
        """Inicialize."""
        if isinstance(parent, QtWidgets.QWidget):
            super().__init__(parent)
        else:
            super().__init__()
            if parent:
                parent.addLayout(self)

        self.setContentsMargins(0, 2, 0, 2)
        self.setSpacing(1)
        self.setSizeConstraint(QtWidgets.QLayout.SizeConstraint.SetMinAndMaxSize)



# -*- coding: utf-8 -*-
# Translated with pineboolib v0.71.18
"""Fltest3_model module."""

import sqlalchemy  # type: ignore [import] # noqa: F821

from pineboolib.application.database.orm import basemodel


class Fltest3(basemodel.BaseModel):  # type: ignore [misc] # noqa: F821
    """Fltest3 class."""

    __tablename__ = "fltest3"

    # --- Metadata --->
    legacy_metadata = {
        "name": "fltest3",
        "alias": "Test table",
        "fields": [
            {
                "name": "counter_field",
                "alias": "Contador",
                "pk": True,
                "type": "string",
                "length": 6,
                "null": False,
                "counter": True,
                "visiblegrid": False,
                "editable": False,
            },
            {"name": "string_field", "alias": "String field", "type": "string", "default": ""},
            {
                "name": "timezone_field",
                "alias": "TimeZone field",
                "type": "timestamp",
                "null": False,
                "default": "2000-01-01 00:00:00",
            },
            {
                "name": "bool_field",
                "alias": "Bool field",
                "type": "bool",
                "null": False,
                "default": False,
                "outtransaction": True,
            },
        ],
    }

    # <--- Metadata ---

    # --- Fields --->

    counter_field = sqlalchemy.Column("counter_field", sqlalchemy.String(6), primary_key=True)
    string_field = sqlalchemy.Column("string_field", sqlalchemy.String)
    timezone_field = sqlalchemy.Column("timezone_field", sqlalchemy.DateTime)
    bool_field = sqlalchemy.Column("bool_field", sqlalchemy.Boolean)


# <--- Fields ---



# -*- coding: utf-8 -*-
"""
MTD Parser to sqlAlchemy model.

Creates a Python file side by side with the original MTD file.
Can be overloaded with a custom class to enhance/change available
functions. See pineboolib/pnobjectsfactory.py
"""

from pineboolib import application, logging
from pineboolib.application import file
import os

from typing import List, Union, TYPE_CHECKING

if TYPE_CHECKING:
    from pineboolib.application.metadata import pnfieldmetadata, pntablemetadata  # pragma: no cover


LOGGER = logging.get_logger(__name__)

RESERVER_WORDS = ["pass"]


def mtd_parse(
    meta_or_name: Union[str, "pntablemetadata.PNTableMetaData"], path_mtd: str = ""
) -> str:
    """
    Parse MTD into SqlAlchemy model.
    """

    if application.PROJECT.conn_manager is None:
        raise Exception("Project is not connected yet")

    dest_file = "%s_model.py" % (
        path_mtd
        if isinstance(meta_or_name, str)
        else "%s/cache/%s" % (application.PROJECT.tmpdir, meta_or_name.name())
    )

    if os.path.exists(dest_file):
        return dest_file

    if isinstance(meta_or_name, str):
        metadata = application.PROJECT.conn_manager.manager().metadata(meta_or_name, True)
    else:
        metadata = meta_or_name

    if metadata is None:
        return ""

    lines = _generate_model(metadata)
    if not lines:
        dest_file = ""
    else:
        _write_file(dest_file, lines)

    return dest_file


def _write_file(file_name: str, lines: List[str]) -> None:
    """Write lines to a file."""

    file_ = open(file_name, "w", encoding="UTF-8")
    file_.writelines([line if line.endswith("\n") else "%s\n" % line for line in lines])
    file_.close()


def _get_meta(file_mtd: "file.File") -> List[str]:
    """Return list with meta."""

    mtd_data_list: List[str] = []
    if os.path.exists(file_mtd.path()):
        mtd_data = application.PROJECT.conn_manager.manager().metadata(file_mtd.filename, True)
        if mtd_data is not None:
            mtd_data_list = _generate_model(mtd_data, False)

    return mtd_data_list


def _generate_model(mtd_table: "pntablemetadata.PNTableMetaData", header: bool = True) -> List[str]:
    """
    Create a list of lines from a mtd_table (pntablemetadata.PNTableMetaData).
    """

    return _create_declaration(mtd_table, header)


def generate_field(field: "pnfieldmetadata.PNFieldMetaData", prev: str = "sqlalchemy") -> str:
    """
    Get text representation for sqlAlchemy of a field type given its pnfieldmetadata.PNFieldMetaData.
    """
    data: List[str] = []
    # TYPE

    # = "String"
    ret = resolve_type(field.type(), field.length(), prev)

    data.append(ret)

    if field.isPrimaryKey():
        data.append("primary_key = True")

    return ", ".join(data)


def resolve_type(type_: str, field_length: int, prev: str) -> str:
    """Return type resolved."""

    ret = ""

    if type_ in ("int, serial"):
        ret = "%s.Integer" % (prev)
    elif type_ in ("uint"):
        ret = "%s.BigInteger" % (prev)
    elif type_ in ("calculated"):
        ret = "%s.String" % (prev)
    # elif field.type() in ("double"):
    #    ret = "sqlalchemy.Numeric"
    #    ret += "(%s , %s)" % (field.partInteger(), field.partDecimal())
    elif type_ == "double":
        ret = "%s.Float" % (prev)

    elif type_ in ("string", "stringlist", "pixmap"):
        ret = "%s.String" % (prev)
        if field_length:
            ret += "(%s)" % field_length

    elif type_ in ("bool", "unlock"):
        ret = "%s.Boolean" % (prev)

    elif type_ == "timestamp":
        ret = "%s.DateTime" % (prev)

    elif type_ == "json":
        ret = "%s.types.JSON" % (prev)

    elif type_ == "time":
        ret = "%s.Time" % (prev)

    elif type_ == "date":
        ret = "%s.Date" % (prev)

    elif type_ in ("bytearray"):
        ret = "%s.LargeBinary" % (prev)

    else:
        ret = "Desconocido %s" % type_

    return ret


def generate_field_metadata(field: "pnfieldmetadata.PNFieldMetaData") -> List[str]:
    """Generate field data from a PNFieldMetaData."""

    field_data: List = []

    # NAME
    field_data.append("'name' : '%s'" % field.name())

    # ALIAS
    if field.alias():
        field_data.append("'alias' : '%s'" % field.alias().replace("'", '"'))

    # PK
    if field.isPrimaryKey():
        field_data.append("'pk' : True")
    # CK
    if field.isCompoundKey():
        field_data.append("'ck' : True")

    # TYPE
    field_relation: List[str] = []
    field_data.append("'type' : '%s'" % field.type())

    # LENGTH
    if field.length():
        field_data.append("'length' : %s" % field.length())

    # REGEXP
    if field.regExpValidator():
        field_data.append("'regexp' : r'%s'" % field.regExpValidator())

    rel_list: List[str]
    # RELATIONS 1M
    for rel in field.relationList():
        rel_list = []
        rel_list.append("'card' : '%s'" % rel.cardinality())
        rel_list.append("'table' : '%s'" % rel.foreignTable())
        rel_list.append("'field' : '%s'" % rel.foreignField())
        if rel.deleteCascade():
            rel_list.append("'delc' : True")
        if rel.updateCascade():
            rel_list.append("'updc' : True")
        if not rel.checkIn():
            rel_list.append("'checkin' : False")

        field_relation.append("{%s}" % ", ".join(rel_list))

    # if field_relation:
    #    field_data.append("'relations' : [%s]" % ", ".join(field_relation))

    # RELATIONS M1
    if field.private._relation_m1:
        rel = field.private._relation_m1
        rel_list = []
        rel_list.append("'card' : '%s'" % rel.cardinality())
        rel_list.append("'table' : '%s'" % rel.foreignTable())
        rel_list.append("'field' : '%s'" % rel.foreignField())
        if rel.deleteCascade():
            rel_list.append("'delC' : True")
        if rel.updateCascade():
            rel_list.append("'updC' : True")
        if not rel.checkIn():
            rel_list.append("'checkIn' : False")

        field_relation.append("{%s}" % ", ".join(rel_list))

    if field_relation:
        field_data.append("'relations' : [%s]" % ", ".join(field_relation))

    # ASSOCIATED
    if field.private.associated_field_name:
        field_data.append(
            "'associated':{'with' : '%s', 'by' : '%s' }"
            % (field.private.associated_field_filter_to, field.private.associated_field_name)
        )

    # UNIQUE
    if field.isUnique():
        field_data.append("'isunique' : True")

    # ALLOW_NULL
    if not field.allowNull():
        field_data.append("'null' : False")

    # DEFAULT_VALUE
    if field.defaultValue() is not None:
        value = (
            field.defaultValue()
            if field.type() in ["bool", "unlock", "int", "uint", "double", "serial", "json"]
            else "'%s'" % field.defaultValue()
        )
        field_data.append("'default' : %s" % value)

    # OUT_TRANSACTION
    if field.outTransaction():
        field_data.append("'outtransaction' : True")

    # COUNTER
    if field.isCounter():
        field_data.append("'counter' : True")

    # CALCULATED
    if field.calculated():
        field_data.append("'calculated' : True")

    # FULLY_CALCULATED
    if field.fullyCalculated():
        field_data.append("'fullycalculated' : True")

    # TRIMMED
    if field.trimmed():
        field_data.append("'trimmed' : True")

    # VISIBLE
    if not field.visible():
        field_data.append("'visible' : False")

    # VISIBLE_GRID
    if not field.visibleGrid():
        field_data.append("'visiblegrid' : False")

    # EDITABLE
    if not field.editable():
        field_data.append("'editable' : False")

    if field.type() == "double":
        # PARTI
        if field.partInteger():
            field_data.append("'partI' : %s" % field.partInteger())

        # PARTD
        if field.partDecimal():
            field_data.append("'partD' : %s" % field.partDecimal())

    # INDEX
    if field.isIndex():
        field_data.append("'index' : True")

    # OPTIONS_LIST
    if field.optionsList():
        texto = ""
        for item in field.optionsList():
            texto += "'%s', " % item

        field_data.append("'optionslist' : [%s]" % texto)
    # SEARCH_OPTIONS
    if field.searchOptions():
        texto = ""
        for item in field.searchOptions():
            texto += "'%s', " % item

        field_data.append("'searchoptions' : [%s]" % texto)

    return field_data


def use_mtd_fields(path_model: str) -> bool:
    """Return if models use mtd fields."""

    file_ = open(path_model, "r", encoding="UTF-8")
    lines = file_.readlines()
    file_.close()
    for line in lines:
        if line.find("legacy_metadata") > -1:
            return False

    return True


def populate_fields(dest_file_name: str, mtd_name: str) -> str:
    """Populate models fields with mtd field."""

    new_file_path: str = ""
    if mtd_name in application.PROJECT.files.keys():
        file_mtd = application.PROJECT.files[mtd_name]

        file_ = open(dest_file_name, "r")
        lines = file_.readlines()
        file_.close()
        new_lines: List[str] = []
        for number, line in enumerate(list(lines)):
            if line.find("__tablename__") > -1:
                new_lines = lines[0 : number + 1] + _get_meta(file_mtd) + lines[number + 1 :]
                break

        if new_lines:
            new_key = "%s_model.py" % file_mtd.filename[:-4]
            conn = application.PROJECT.conn_manager.mainConn()
            db_name = conn.DBName()
            application.PROJECT.files[new_key] = file.File(
                file_mtd.module,
                "%s_model.py" % file_mtd.path(),
                basedir=file_mtd.basedir,
                sha=file_mtd.sha,
                db_name=db_name,
            )
            application.PROJECT.files[new_key].filekey = "%s_model.py" % file_mtd.filekey
            new_file_path = application.PROJECT.files[new_key].path()
            if os.path.exists(new_file_path):
                os.remove(new_file_path)

            _write_file(new_file_path, new_lines)

    return new_file_path


def _create_declaration(
    mtd_table: "pntablemetadata.PNTableMetaData", header: bool = True
) -> List[str]:
    """Create metadata section."""

    data: List[str] = []
    list_data_field: List[str] = []
    validator_list: List[str] = []
    metadata_table: List = []
    metadata_table.append("'name' : '%s'" % mtd_table.name())
    metadata_table.append("'alias' : '%s'" % mtd_table.alias())
    cached_fields = mtd_table.cachedFields()
    if cached_fields:
        LOGGER.warning("Adding cachedfields '%s' to model %s" % (cached_fields, mtd_table.name()))
        metadata_table.append("'cachedfields' : '%s'" % cached_fields)

    if mtd_table.isQuery():
        metadata_table.append("'query':'%s'" % mtd_table.query())
    if mtd_table.concurWarn():
        metadata_table.append("'concurwarn': True")
    if mtd_table.detectLocks():
        metadata_table.append("'detectlocks':True")
    if mtd_table.FTSFunction():
        metadata_table.append("'ftsfunction' :'%s'" % mtd_table.FTSFunction())

    try:
        mtd_table.primaryKey()
    except Exception as error:  # noqa: F841
        pass

    field_list: List[List[str]] = []
    pk_found = False

    for field in mtd_table.fieldList():  # Crea los campos
        if field.isPrimaryKey():
            pk_found = True

        if field.name() in validator_list:
            LOGGER.warning(
                "Hay un campo %s duplicado en %s.mtd. Omitido", field.name(), mtd_table.name()
            )
        else:
            field_data = []
            field_data.append("    ")
            if field.name() in RESERVER_WORDS:
                field_data.append("%s_" % field.name())
            else:
                field_data.append(field.name())

            field_data.append(" = sqlalchemy.Column('%s', " % field.name())
            field_list.append(generate_field_metadata(field))
            field_data.append(generate_field(field))
            field_data.append(")")
            validator_list.append(field.name())
            if field.isPrimaryKey():
                pk_found = True

        list_data_field.append("".join(field_data))

    meta_fields: List = []
    for meta_field in field_list:
        meta_fields.append("{%s}" % ", ".join(meta_field))

    metadata_table.append(
        "\n        'fields' : [\n        %s\n        ]" % ",\n        ".join(meta_fields)
    )

    class_name = "%s%s" % (mtd_table.name()[0].upper(), mtd_table.name()[1:])

    if header:
        data.append("# -*- coding: utf-8 -*-")
        data.append("# Translated with pineboolib %s" % application.PINEBOO_VER)
        data.append(
            '"""%s%s_model module."""' % (mtd_table.name()[0].upper(), mtd_table.name()[1:])
        )
        data.append("")

        data.append("from pineboolib.application.database.orm import basemodel")
        data.append("from pineboolib.qsa import qsa")
        data.append("")
        data.append("import sqlalchemy")
        data.append("")
        data.append("")

        data.append("# @class_declaration Oficial")
        data.append("class Oficial(basemodel.BaseModel): # type: ignore [misc] # noqa: F821")
        data.append('    """Oficial class."""')
        data.append("    __tablename__ = '%s'" % mtd_table.name())  # si query nombre query
        data.append("")
    else:
        data.append("")
        data.append("")
        data.append("    # --- POPULATED WITH METADATA FIELDS ---")
        data.append("")
        data.append("")
    data.append("    # --- Metadata --->")
    data.append("    legacy_metadata = {%s}" % ", ".join(metadata_table))
    data.append("\n")
    data.append("    # <--- Metadata ---")

    data.append("")

    data.append("")
    data.append("    # --- Fields --->")
    data.append("")

    for data_field in list_data_field:
        data.append(data_field)

    data.append("")
    data.append("    # <--- Fields ---")

    data.append("")
    if header:
        data.append("# @class_declaration %s" % class_name)
        data.append(
            "class %s(Oficial): # type: ignore [misc] # noqa: F821" % class_name
        )  # si query nombre query
        data.append('    """%s class."""' % class_name)
        data.append("    pass")

    if not pk_found and not mtd_table.isQuery():
        LOGGER.warning(
            "La tabla %s no tiene definida una clave primaria. No se generará el model."
            % (mtd_table.name())
        )
        data = []

    return data



"""Qtimeedit module."""
# -*- coding: utf-8 -*-
from PyQt6 import QtWidgets, QtCore  # type: ignore[import]
from typing import Optional, Union


class QTimeEdit(QtWidgets.QTimeEdit):
    """QTimeEdit class."""

    def __init__(self, parent: Optional["QtWidgets.QWidget"] = None) -> None:
        """Inicialize."""
        super().__init__(parent)

        self.setDisplayFormat("hh:mm:ss A")

    def setTime(  # type: ignore [override] # noqa: F821
        self, time: Union["QtCore.QTime", str]
    ) -> None:
        """Set time."""
        if not isinstance(time, QtCore.QTime):
            t_list = time.split(":")
            time = QtCore.QTime(int(t_list[0]), int(t_list[1]), int(t_list[2]))
        super().setTime(time)

    def getTime(self) -> str:
        """Return time."""

        return super().time().toString("hh:mm:ss")

    time: str = property(getTime, setTime)  # type: ignore [assignment] # noqa F821



"""Test_qbytearray module."""

import unittest


class TestQByteArray(unittest.TestCase):
    """TestQByteArray Class."""

    def test_all(self) -> None:
        """Test qbytearray."""
        import base64
        from pineboolib.qsa import qsa

        text_64 = base64.b64encode("hola holita!".encode("UTF-8"))

        ba_ = qsa.QByteArray(len(text_64))
        i = 0
        for data in text_64:
            ba_.set(i, data)
            i += 1

        ba2 = qsa.QByteArray(ba_.fromBase64())

        self.assertEqual(ba_.data(), text_64)
        self.assertEqual(ba2.data(), b"hola holita!")
        self.assertEqual(chr(ba2.get(3)), "a")



"""Test DlgConnect module."""

import unittest
from pineboolib.loader.main import init_testing, finish_testing
from pineboolib.loader.dlgconnect import dlgconnect
from pineboolib.core import settings
from pineboolib.core.utils import utils_base
import os
from pathlib import Path

actual_dir_profile: str = ""


class TestDlgConnect(unittest.TestCase):
    """Tes DlgConnect class."""

    @classmethod
    def setUpClass(cls) -> None:
        """Ensure pineboo is initialized for testing."""

        init_testing()

    def test_basic_1(self) -> None:
        """Test basic 1."""

        dialog = dlgconnect.DlgConnect()
        dialog.load()
        dialog.cleanProfileForm()

        # dialog._user_interface.cbDBType.
        dialog._user_interface.cbProfiles.setEditText("eoo")
        dialog.edit_mode = False
        dialog._user_interface.cbAutoLogin.setChecked(True)
        dialog._user_interface.leDescription.setText("pytest")
        dialog._user_interface.leDBPassword.setText("12345")
        dialog._user_interface.leDBPassword2.setText("12345")
        dialog._user_interface.leProfilePassword.setText("12345")
        dialog._user_interface.leProfilePassword2.setText("12345")
        dialog._user_interface.leDBName.setText("prueba_db")
        dialog._user_interface.leURL.setText("localhost")
        dialog._user_interface.lePort.setText("5432")
        dialog._user_interface.leDBUser.setText("usuario")
        dialog._user_interface.leProfilePassword.setText("12345")
        self.assertEqual(dialog._user_interface.leURL.text(), "localhost")
        self.assertEqual(dialog._user_interface.leDBUser.text(), "usuario")
        self.assertEqual(dialog._user_interface.leDBPassword.text(), "12345")
        self.assertEqual(dialog._user_interface.leDBName.text(), "prueba_db")
        self.assertEqual(dialog._user_interface.leProfilePassword.text(), "12345")
        profile_dir: str = utils_base.filedir(
            settings.CONFIG.value(
                "ebcomportamiento/profiles_folder", "%s/Pineboo/profiles" % Path.home()
            )
        )
        if os.path.exists("%s/pytest.xml" % profile_dir):
            os.remove("%s/pytest.xml" % profile_dir)
        dialog.saveProfile()
        dialog.showOptions(True)
        dialog.showOptions(False)
        self.assertTrue(dialog.getProjectConfig("pytest"))
        dialog.editProfileName("pytest")
        dialog.close()

    @classmethod
    def tearDownClass(cls) -> None:
        """Ensure this class is finished correctly."""

        finish_testing()



"""Test_aqods module."""

import unittest
from pineboolib.qsa import qsa


class TestAQOds(unittest.TestCase):
    """TestAQOds Class."""

    def test_aq_ods_color(self) -> None:
        """Test AQOdsColor."""

        val_1 = qsa.AQOdsColor(0xE7E7E7)
        val_2 = qsa.AQOdsColor(242, 150, 141)

        self.assertEqual(val_1, "e7e7e7")
        self.assertEqual(val_2, "f2968d")

    def test_aq_ods_row(self) -> None:
        """Test AQOdsRow."""

        generator = qsa.AQOdsGenerator()
        hojas = qsa.AQOdsSpreadSheet(generator)
        hoja = qsa.AQOdsSheet(hojas, "hoja_1")
        row_1 = qsa.AQOdsRow(hoja)
        self.assertTrue(row_1)
        row_1.setFixedPrecision(1)
        row_1.opIn(0.21)
        row_2 = qsa.AQOdsRow(hoja)
        self.assertTrue(row_2)
        style_obj = qsa.AQOdsStyle(19)

        style_obj.alignCenter()
        style_obj.alignLeft()
        style_obj.alignRight()
        style_obj.textBold()
        style_obj.textItalic()
        style_obj.textUnderline()
        style_obj.borderBottom()
        style_obj.borderLeft()
        style_obj.borderRight()
        style_obj.borderTop()

        self.assertTrue(style_obj)
        row_2.opIn(style_obj)
